/*
 * systest.c -- Test code for nooks system calls
 *
 * Copyright (C) 2002 Mike Swift
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  
 * No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
//#include <unistd.h>
#include <string.h>
//#include <alloca.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>
#include "nooks-sys.h"
#include "msr.h" /* actually asm/msr.h */
#include <sys/time.h>
//#include <stddef.h>
//#include <getopt.h>
//#include <sys/stat.h>
//#include <sys/file.h>

#define NOOKS_QUERY_STATE               1
#define __NR_nooks_create_nook  318
#define __NR_nooks_delete_nook  319
#define __NR_nooks_query_nook   320
#define __NR_nooks_enum_nooks   321
#define __NR_nooks_ioctl        322
#define __NR_nooks_unload_nook  323
#define __NR_nooks_replug_module 324
#define __NR_swifi_inject_fault 325

typedef enum actions {
  no_action = 0,
  create_nook,
  delete_nook,
  query_nook,
  enum_nooks, 
  ioctl_nook,
  unload_nook,
  inject_fault
} actions_t;



int
main(int argc, char * argv[])
{
  char * nook_name = NULL;
  int i;
  long result = 0;
  actions_t action = no_action;
  unsigned int cmd = 0;
  unsigned long arg = 0;
  unsigned long seed = 157;

  if (argc < 2) {
    goto Usage;
  }

  for (i = 1; i < argc; i++ ) {
    if (strcmp(argv[i], "-c") == 0) {
      if (argc <= i+1) {
	goto Usage;
      }
      nook_name = argv[++i];
      action = create_nook;
    } else if (strcmp(argv[i], "-d") == 0) {
      if (argc <= i+1) {
	goto Usage;
      }
      nook_name = argv[++i];
      action = delete_nook;
    } else if (strcmp(argv[i], "-u") == 0) {
      if (argc <= i+1) {
	goto Usage;
      }
      nook_name = argv[++i];
      action = unload_nook;
    } else if (strcmp(argv[i], "-q") == 0) {
      if (argc <= i+1) {
	goto Usage;
      }
      nook_name = argv[++i];
      action = query_nook;
    } else if (strcmp(argv[i], "-f") == 0) {
      if (argc <= i+4) {
	goto Usage;
      }
      nook_name = argv[++i];
      action = inject_fault;
      sscanf(argv[++i],"%u", &cmd);
      sscanf(argv[++i],"%lu", &arg);
      sscanf(argv[++i],"%lu", &seed);
    } else if (strcmp(argv[i], "-i") == 0) {
      if (argc <= i+3) {
	goto Usage;
      }
      nook_name = argv[++i];
      action = ioctl_nook;
      sscanf(argv[++i],"%u", &cmd);
      sscanf(argv[++i],"%lu", &arg);
    } else if (strcmp(argv[i], "-i2") == 0) {
      int arg1, arg2;
      if (argc <= i+4) {
	goto Usage;
      }
      nook_name = argv[++i];
      action = ioctl_nook;
      sscanf(argv[++i],"%u", &cmd);
      sscanf(argv[++i],"%d", &arg1);
      sscanf(argv[++i],"%d", &arg2);
      arg = NOOKS_MAKE_ARG(arg1, arg2, 16);
    } else if (strcmp(argv[i], "-e") == 0) {
      action = enum_nooks;
    } else {
      printf("Unknown command %s\n", argv[i]);
      goto Usage;
    }
  }
  if (action == no_action) {
    printf("No action specified.\n");
    goto Usage;
  }

  switch(action) {
  case create_nook:

    result = nooks_create_nook(nook_name);
    printf("nooks_create_nook returned %ld (%d)\n", result, errno);

    break;
  case delete_nook:
    result = nooks_delete_nook(nook_name);
    printf("nooks_delete_nook returned %ld (%d)\n", result, errno);
    break;
  case unload_nook:
    result = nooks_unload_nook(nook_name);
    printf("nooks_unload_nook returned %ld (%d)\n", result, errno);
    break;
  case query_nook:
    {
      size_t size = 0;
      result = nooks_query_nook(nook_name, NOOKS_QUERY_STATE,NULL, size, &size);
      printf("nooks_query_nook returned %ld (%d)\n", result,errno);	
      break;
    }
  case ioctl_nook:
    {
      struct timeval timestamp1,timestamp2;
      gettimeofday(&timestamp1, NULL);
      result = nooks_ioctl(nook_name, cmd, arg);
      gettimeofday(&timestamp2, NULL);
      printf("Ioctl at time %ld:%ld\n", 
	     timestamp1.tv_sec, timestamp1.tv_usec);  
      printf("Done ioctl at time %ld:%ld\n", 
	     timestamp2.tv_sec, timestamp2.tv_usec);  
      printf("nooks_ioctl_nook returned %ld (%d)\n", result,errno);	
      break;
    }
  case inject_fault:
    {
      struct timeval timestamp1,timestamp2;
      swifi_result_t res[arg];
      int i;

      memset(res, 0, sizeof(res));

      //
      // Find out where the faults will be injected
      //

      result = swifi_inject_fault(nook_name, 
				  cmd,         // fault type
				  seed,           // random seed
				  arg,         // numFaults
				  res,
				  0);  // don't inject now

      for (i = 0; (i < arg) && (res[i].address != 0) ; i++) {
	printf("Changed 0x%lx from 0x%lx to 0x%lx\n",
	       res[i].address,
	       res[i].old,
	       res[i].new);
      }

      //
      // do the injection
      //

      gettimeofday(&timestamp1, NULL);

      result = swifi_inject_fault(nook_name, 
				  cmd,         // fault type
				  seed,           // random seed
				  arg,         // numFaults
				  res,
				  1);  // do inject now

      gettimeofday(&timestamp2, NULL);
      printf("Injecting at time %ld:%ld\n", 
	     timestamp1.tv_sec, timestamp1.tv_usec);  
      printf("Done injecting at time %ld:%ld\n", 
	     timestamp2.tv_sec, timestamp2.tv_usec);  
      printf("swifi_inject_fault returned %ld (%d)\n", result,errno);

      break;
    }
  case enum_nooks:
    {
      char * buf = NULL;
      size_t size = 0;
      size = 0;
      //      buf = (char *) malloc(500);
      printf ("Calling nooks_enum_nooks.\n");
      result = nooks_enum_nooks(buf, size, &size);
      if ((result < 0) && (errno == ENOSPC)) {
	buf = (char *) malloc(size);
	if (buf != NULL) {
	  result = nooks_enum_nooks(buf, size, &size);
	} else {
	  printf("Failed to allocate size=%d bytes\n",size);
	  result = -ENOMEM;
	}
      } else {
	printf("nooks_enum_nooks returned %ld (%d)\n", result, errno);
      }
      if (result == 0) {
	char * next = buf;
	while (*next != 0) {
	  printf("%s\n",next);
	  next += strlen(next) +1;
	}	  
      }
      if (buf != NULL) {
	free(buf);
      }

      break;
    }
  case no_action:
  	break;  
  }


 Cleanup:
  return(0);

 Usage:
  printf("Usage: %s [-c|-d|-q|-e|-i|-f] nookname\n", argv[0]);
  goto Cleanup;
}


